@@ -1,3 +1,17 @@
+# Copyright 2015 Oliver Cope
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
+#
 """
 fresco.request
 --------------
@@ -5,13 +19,14 @@ fresco.request
 The :class:`Request` class models an incoming HTTP request, allowing access to
 HTTP headers and request data (eg query string or submitted form data).
 """
+from __future__ import absolute_import
 
 import json
 import posixpath
 import re
 
 from fresco import exceptions
-from fresco.compat import (ustring_type, quote, urlparse, urlunparse,
+from fresco.compat import (ustr, quote, urlparse, urlunparse,
                            ParseResult)
 from fresco.util.wsgi import environ_to_unicode
 
@@ -98,6 +113,7 @@ class Request(object):
     _files = None
     _query = None
     _cookies = None
+    _parsed_url = None
 
     #: WSGI key for the session variable. The default value is configured for
     #: use with `beaker <http://beaker.groovie.org/>`_
@@ -204,7 +220,10 @@ class Request(object):
         This will try to decode a json string regardless of the mime type sent
         by the client.
         """
-        return self.json_decoder_class(*args, **kwargs).decode(self.body)
+        try:
+            return self.json_decoder_class(*args, **kwargs).decode(self.body)
+        except ValueError:
+            raise exceptions.RequestParseError("Payload is not valid JSON")
 
     @property
     def files(self):
@@ -276,25 +295,22 @@ class Request(object):
 
             >>> from fresco import FrescoApp
             >>> with FrescoApp().requestcontext(
-            ...     HTTP_COOKIE='''$Version="1";
-            ...     Customer="WILE_E_COYOTE";
+            ...     HTTP_COOKIE='''Customer="WILE_E_COYOTE";
             ...     Part="Rocket_0001";
             ...     Part="Catapult_0032"
             ... ''') as c:
             ...     request = c.request
             ...
-            >>> [item.value for item in request.cookies.getlist('Customer')]
+            >>> request.cookies.getlist('Customer')
             ['WILE_E_COYOTE']
-            >>> [item.value for item in request.cookies.getlist('Part')]
+            >>> request.cookies.getlist('Part')
             ['Rocket_0001', 'Catapult_0032']
 
         See rfc2109, section 4.4
         """
         if self._cookies is None:
             self._cookies = MultiDict(
-                (cookie.name, cookie)
-                for cookie in parse_cookie_header(self.get_header("Cookie"))
-            )
+                parse_cookie_header(self.environ.get('HTTP_COOKIE', '')))
         return self._cookies
 
     def get_header(self, name, default=None):
@@ -375,6 +391,9 @@ class Request(object):
 
         Components are returned as unicode strings
         """
+        if self._parsed_url:
+            return self._parsed_url
+
         env = self.environ.get
         script_name = quote(self.script_name.encode(self.charset))
         path_info = quote(self.path_info.encode(self.charset))
@@ -400,14 +419,14 @@ class Request(object):
         else:
             netloc = host + ':' + port
 
-        return ParseResult(
-            scheme,
-            netloc,
-            script_name + path_info,
-            '',  # Params
-            query_string,
-            '',  # Fragment
-        )
+        self._parsed_url = ParseResult(scheme,
+                                       netloc,
+                                       script_name + path_info,
+                                       '',  # Params
+                                       query_string,
+                                       '',  # Fragment
+                                       )
+        return self._parsed_url
 
     @property
     def path_info(self):
@@ -482,6 +501,13 @@ class Request(object):
         """
         return self.environ.get(self.SESSION_ENV_KEY)
 
+    @property
+    def is_secure(self):
+        """
+        Return ``True`` if the request is served over a secure connection.
+        """
+        return self.environ['wsgi.url_scheme'] == 'https'
+
     def make_url(self, scheme=None, netloc=None, path=None, parameters=None,
                  query=None, query_add=None, query_replace=None, fragment=None,
                  SCRIPT_NAME=None, PATH_INFO=None, **kwargs):
@@ -611,7 +637,7 @@ class Request(object):
             path = parsed_url[2]
 
         if query is not None:
-            if isinstance(query, ustring_type):
+            if isinstance(query, ustr):
                 # Unicode queries should be pre-encoded and thus contain ascii
                 # characters only
                 query = query.encode('ASCII')
