@@ -1,3 +1,19 @@
+# Copyright 2015 Oliver Cope
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
+#
+from __future__ import absolute_import
+
 from operator import attrgetter, itemgetter, methodcaller
 from fresco.exceptions import MissingRouteArg
 from itertools import cycle
@@ -71,7 +87,7 @@ class RouteArg(object):
 
             def __call__(self, request):
                 try:
-                    return request.cookies[self.name].value
+                    return request.cookies[self.name]
                 except KeyError,
                     return None
 
@@ -85,6 +101,11 @@ class RouteArg(object):
     route = None
     name = None
 
+    def __init__(self, default=_marker, exception=MissingRouteArg):
+        self.default = default
+        self.required = default is _marker
+        self.exception = exception
+
     def configure(self, route, name):
         self.route = route
         self.name = name
@@ -96,14 +117,14 @@ class RouteArg(object):
 def routearg(func, *args, **kwargs):
     """
     Construct a ``RouteArg`` instance that calls ``func`` with the request
-    object as an argument and passes the return value to the routed view.
+    object as an argument and passes the return value to the routed view::
 
         def extract_name(req):
             return req.get('name')
 
         Route('/', GET, myview, name=routearg(extract_name))
 
-    Additional positional and keyword arguments may be passed, eg:
+    Additional positional and keyword arguments may be passed, eg::
 
         def name_is(req, *friends):
             return req.get('name') in friends
@@ -136,11 +157,8 @@ class RequestArg(RouteArg):
 
     def __init__(self, converter=None, key=None,
                  default=_marker, exception=MissingRouteArg):
-
+        super(RequestArg, self).__init__(default, exception)
         self.formkey = key
-        self.default = default
-        self.required = default is _marker
-        self.exception = exception
         self.is_list = isinstance(converter, list)
         if self.is_list:
             self.converter = lambda vs: [c(v)
@@ -192,6 +210,24 @@ class RequestObject(RouteArg):
         return request
 
 
+class JSONPayload(RouteArg):
+    """
+    Make the JSON payload from the request available as a view argument.
+
+    Example::
+
+        @app.route('/form', POST, data=JSONPayload())
+        def view(formdata, request):
+            ...
+    """
+    def __call__(self, request):
+        if request.content_type == 'application/json':
+            return request.get_json()
+        if self.default is not _marker:
+            return self.default
+        return LazyException(self.exception('JSON payload missing'))
+
+
 class GetData(RouteArg):
     """\
     Make the ``request.form`` MultiDict available as a view argument.
@@ -243,9 +279,9 @@ class CookieArg(RequestArg):
 
     def getter(self, cookies):
         if self.is_list:
-            return [c.value for c in cookies.getlist(self.formkey)]
+            return cookies.getlist(self.formkey)
         else:
-            return cookies[self.formkey].value
+            return cookies[self.formkey]
 
     def configure(self, route, name):
         if self.formkey is None:
