@@ -1,13 +1,31 @@
+# Copyright 2015 Oliver Cope
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
+#
+from __future__ import absolute_import
+
 import re
 import sys
 from copy import copy
-from collections import namedtuple
+from collections import defaultdict, namedtuple, Iterable
+from weakref import WeakKeyDictionary
 
 from fresco.compat import string_types
 from fresco.exceptions import ResponseException
 from fresco.response import Response
 from fresco.requestcontext import context
 from fresco.routeargs import RouteArg
+from fresco.util.common import fq_path
 from fresco.util.urls import join_path
 
 RFC2518_METHODS = \
@@ -33,9 +51,9 @@ RFC5323_METHODS = SEARCH, = ('SEARCH',)
 
 RFC5789_METHODS = PATCH, = ('PATCH',)
 
-ALL_METHODS = HTTP_METHODS = (RFC2518_METHODS + RFC2616_METHODS +
-                              RFC3253_METHODS + RFC3648_METHODS +
-                              RFC5323_METHODS + RFC5789_METHODS)
+ALL_METHODS = HTTP_METHODS = set(RFC2518_METHODS + RFC2616_METHODS +
+                                 RFC3253_METHODS + RFC3648_METHODS +
+                                 RFC5323_METHODS + RFC5789_METHODS)
 
 __all__ = ('ALL_METHODS', 'Pattern', 'Route', 'DelegateRoute',
            'RouteCollection', 'routefor')
@@ -47,10 +65,85 @@ __all__ += tuple(method.replace('-', '_') for method in ALL_METHODS)
 PathMatch = namedtuple('PathMatch', ['path_matched', 'path_remaining',
                                      'args', 'kwargs'])
 
-#: Encapsulate a route traversal
-RouteTraversal = namedtuple('RouteTraversal',
-                            ['route', 'args', 'kwargs',
-                             'collections_traversed'])
+
+class RouteTraversal(namedtuple('RouteTraversal',
+                                'route args kwargs collections_traversed')):
+    """
+    Encapsulate a route traversal.
+
+    Each ``RouteTraversal`` object contains:
+
+    - ``route`` the final route traversed, allowing access to the view
+        associated with the path.
+    - ``args`` - positional args to be passed to the view. This is a
+        combination of args extracted from the path and any added when
+        constructing the route.
+    - ``kwargs`` - keyword args to be passed to the view. This is a
+        combination of kwargs extracted from the path and any added when
+        constructing the route.
+    -  ``collections_traversed`` - a list of ``TraversedCollection``
+        objects, containing information on which RouteCollections were
+        encountered during traversal, the route selected at each stage
+        and any args/kwargs associated with that phase of the traversal.
+    """
+
+    def replace(self, viewspec, traversal_kwargs=None, **kwargs):
+        """
+        Return a new ``RouteTraversal`` with the traversal_kwargs or other
+        fields replaced for the route identified by ``viewspec``.
+
+        Example:
+
+            >>> from fresco import FrescoApp
+            >>> app = FrescoApp()
+            >>> @route('/<lang:str>/index.html', GET, homepage, name='home')
+            ... def homepage(lang):
+            ...     return Response({'fr': 'Bonjour!', 'en': 'Hello!'}[lang])
+            ...
+            >>> traversal = next(app.get_routes('/fr/index.html'))
+            >>> en_traversal = traversal.replace('home', {'lang': 'en'})
+            >>> en_traversal.build_path()
+            '/en/index.html'
+
+        """
+        if traversal_kwargs is not None:
+            kwargs['traversal_kwargs'] = traversal_kwargs
+        if isinstance(viewspec, string_types):
+            viewspecs = viewspec.split(':')
+        else:
+            viewspecs = [viewspec]
+        collections_traversed = iter(self.collections_traversed)
+        for item in viewspecs:
+            while True:
+                ct = next(collections_traversed, None)
+                if ct is None:
+                    raise RouteNotFound(viewspec)
+                try:
+                    route = ct.collection.routefor(item)
+                    break
+                except RouteNotFound:
+                    continue
+
+        collections_traversed = [c._replace(**kwargs)
+                                 if (c.route is route) else c
+                                 for c in self.collections_traversed]
+        return self._replace(collections_traversed=collections_traversed)
+
+    def build_path(self):
+        """
+        Reconstruct the path from a route traversal
+        """
+        paths = []
+        for c in self.collections_traversed:
+            path, _, _ = c.route.path(*c.traversal_args,
+                                      **c.traversal_kwargs)
+            paths.append(path)
+        return ''.join(paths)
+
+#: An item of RouteTraversal.collections_traversed
+TraversedCollection = namedtuple('TraversedCollection',
+                                 'collection path route args kwargs '
+                                 'traversal_args traversal_kwargs')
 
 
 _marker = object()
@@ -509,9 +602,9 @@ class Route(object):
     #: The default class to use for URL pattern matching
     pattern_class = ExtensiblePattern
 
-    def __init__(self, pattern, methods, view, kwargs=None,
+    def __init__(self, pattern, methods=None, view=None, kwargs=None,
                  args=None, name=None, predicate=None, decorators=None,
-                 **_kwargs):
+                 filters=None, **_kwargs):
 
         """
         :param pattern:     A string that can be compiled into a path pattern
@@ -528,9 +621,13 @@ class Route(object):
                             object as its only parameter and return a boolean.
         :param decorators:  Decorator functions to apply to the view callable
                             before invoking it
-        :param **_kwargs:   Other keyword aruments passed through to the view
-                            callable.
-
+        :param filters:     Filter functions to apply to the view's return
+                            value before returning the final response object
+        :param **_kwargs:   Keyword arguments matching HTTP method names
+                            (GET, POST etc) can used to specify views
+                            associated with those methods.
+                            Other keyword aruments are passed through to the
+                            view callable.
 
         Naming routes
         -------------
@@ -572,15 +669,24 @@ class Route(object):
             'http://localhost/thumbnail'
 
         """
+        method_view_map = {}
+        if methods:
+            if isinstance(methods, string_types):
+                methods = [methods]
+            else:
+                # Catch the common error of not providing a valid method
+                if not isinstance(methods, Iterable):
+                    raise TypeError("HTTP methods must be specified as a "
+                                    "string or iterable")
+            for m in methods:
+                if m not in ALL_METHODS:
+                    raise ValueError(
+                        "{!r} is not a valid HTTP method".format(m))
+            method_view_map.update((m, view) for m in methods)
 
-        if isinstance(methods, string_types):
-            methods = [methods]
-        else:
-            # Catch the common error of not providing a valid method
-            try:
-                methods = set(methods)
-            except TypeError:
-                raise TypeError("Methods must be a string or iterable")
+        for method in ALL_METHODS:
+            if method in _kwargs:
+                method_view_map[method] = _kwargs.pop(method)
 
         if not isinstance(pattern, Pattern):
             pattern = self.pattern_class(pattern)
@@ -592,18 +698,22 @@ class Route(object):
         self.name = name
         self.predicate = predicate
         self.decorators = decorators or []
-        self.filters = []
+        self.filters = filters or []
 
         #: Default values to use for path generation
         self.routed_args_default = {}
 
         self.pattern = pattern
-        self.methods = set(methods)
+        self.methods = set(method_view_map)
         self.instance = None
-        # Cached reference to the view function
-        self._view = None
-        # Cached reference to the decorated view function
-        self._dview = None
+
+        # Cached references to view functions
+        self._cached_views = {}
+
+        # Cached references to decorated view function. We use weakrefs in case
+        # a process_view hook substitutes the view function used as a key
+        # for a dynamically generated function.
+        self._cached_dviews = WeakKeyDictionary()
 
         p_args, p_kwargs = pattern.path_argument_info()
         for k in p_kwargs:
@@ -626,65 +736,70 @@ class Route(object):
             if isinstance(arg, RouteArg):
                 arg.configure(self, argname)
 
-        self.viewspec = view
-
-    def __str__(self):
-        if self.methods == ALL_METHODS:
-            methods = '*'
-        else:
-            methods = ' '.join(self.methods)
-
-        return '%s %s => %r\n' % (
-            methods,
-            str(self.pattern),
-            self.viewspec
-        )
+        #: A mapping of HTTP methods to view specifiers
+        self.viewspecs = method_view_map
 
     def __repr__(self):
-        if self.methods == ALL_METHODS:
-            methods = '*'
-        else:
-            methods = ','.join(self.methods)
-        return '<%s %s %r %s>' % (
-            self.__class__.__name__,
-            methods,
-            str(self.pattern),
-            self.viewspec
-        )
+        view_methods_map = defaultdict(set)
+        for method, viewspec in self.viewspecs.items():
+            view_methods_map[viewspec].add(method)
+
+        s = []
+        for viewspec, methods in view_methods_map.items():
+            if methods == ALL_METHODS:
+                methods = '*'
+            else:
+                methods = ' '.join(self.methods)
+            s.append('%s %s => %r' % (methods, str(self.pattern), fq_path(viewspec)))
+
+        return '<%s %s>' % (self.__class__.__name__,
+                            '\n      '.join(s))
+
+    def __getstate__(self):
+        state = self.__dict__.copy()
+        state['_cached_views'] = {}
+        state['_cached_dviews'] = WeakKeyDictionary()
+        return state
 
     def match(self, path, method):
         if method and method not in self.methods:
             return None
         return self.pattern.match(path)
 
-    def getview(self):
+    def getview(self, method):
         """\
         Return the raw view callable.
         """
-        if self._view:
-            return self._view
-        uview = self.viewspec
+        try:
+            return self._cached_views[method]
+        except KeyError:
+            pass
+
+        uview = self.viewspecs[method]
         if isinstance(uview, string_types):
             if self.instance is None:
                 raise RouteNotReady()
             uview = getattr(self.instance, uview)
 
-        self._view = uview
+        self._cached_views[method] = uview
         return uview
 
-    def getdecoratedview(self):
+    def getdecoratedview(self, view):
         """\
         Return the view callable decorated with any decorators defined in the
         route
         """
-        if self._dview:
-            return self._dview
-        dview = self.getview()
+        try:
+            return self._cached_dviews[view]
+        except KeyError:
+            pass
+
+        dview = view
         for d in (self.decorators or []):
             dview = d(dview)
         if self.filters:
             dview = self._apply_filters(dview)
-        self._dview = dview
+        self._cached_dviews[view] = dview
         return dview
 
     def _apply_filters(self, view):
@@ -705,9 +820,7 @@ class Route(object):
 
         Use this when traversing view classes.
         """
-        cls = self.__class__
-        ob = cls.__new__(cls)
-        ob.__dict__ = self.__dict__.copy()
+        ob = copy(self)
         ob._setinstance(instance)
         return ob
 
@@ -726,11 +839,14 @@ class Route(object):
 
     def path(self, *args, **kwargs):
         """\
-        Return a tuple of URL path component corresponding to the route and
-        remaining positional and keyword arguments
+        Build the path corresponding to this route and return a tuple of:
 
-        Positional and keywords arguments are forwarded to the ``pathfor``
-        method of the pattern.
+            ``(<path>, <remaining args>, <remaining kwargs>)``
+
+        ``remaining args`` and ``remaining kwargs`` are any values from
+        ``*args`` and ``**kwargs`` not consumed during path construction.
+
+        See also :meth:`~fresco.routing.Pattern.pathfor`.
         """
         request = kwargs.pop('request', None)
         for k in self.routed_args_default:
@@ -747,18 +863,20 @@ class Route(object):
         """
         if self.name:
             yield self.name
-        yield self.viewspec
-        try:
-            view = self.getview()
-        except RouteNotReady:
-            return
-        yield view
 
-        # Also return the underlying function object for python2 unbound
-        # methods
-        func_ob = getattr(view, '__func__', None)
-        if func_ob is not None:
-            yield func_ob
+        for method, viewspec in self.viewspecs.items():
+            yield viewspec
+            try:
+                view = self.getview(method)
+            except RouteNotReady:
+                continue
+            yield view
+
+            # Also return the underlying function object for python2 unbound
+            # methods
+            func_ob = getattr(view, '__func__', None)
+            if func_ob is not None:
+                yield func_ob
 
     def wrap(self, *decorators):
         """
@@ -825,51 +943,49 @@ class RouteCollection(object):
         self.route_class = route_class or self.route_class
         if routes is not None:
             for item in routes:
-                self.add_route(item)
+                if isinstance(item, Route):
+                    self.add_route(item)
+                elif hasattr(item, '__routes__'):
+                    for r in item.__routes__:
+                        if r.instance is None:
+                            r = r.bindto(item)
+                        self.add_route(r)
+                elif isinstance(item, Iterable):
+                    for r in item:
+                        self.add_route(r)
+                else:
+                    raise TypeError(item)
 
     def __add__(self, other):
         result = copy(self)
-        for item in other:
-            result.add_route(item)
+        if isinstance(other, Route):
+            result.add_route(other)
+        else:
+            for item in other:
+                result.add_route(item)
         return result
 
+    def __radd__(self, other):
+        if isinstance(other, Route):
+            return RouteCollection([other]) + self
+        elif isinstance(other, Iterable):
+            return RouteCollection(other) + self
+        raise TypeError("Cannot add %r to %r" % (other, self))
+
     def __iter__(self):
         return iter(self.__routes__)
 
     def add_route(self, route):
         self.__routes__.append(route)
-        try:
-            view = route.getview()
-        except RouteNotReady:
-            # If view methods are referenced by name in a class's __routes__
-            # attribute, and the class has not yet been instantiated, this will
-            # fail. However the view.url method is now deprecated, so we
-            # can ignore this case.
-            pass
-        else:
-            self._add_url_method(view, route)
 
-    def _add_url_method(self, func, route):
-        """\
-        Add a method available at ``func.url`` that returns a URL generated
-        from the given route
+    def add_prefix(self, prefix):
         """
-        if hasattr(func, 'url'):
-            return
-
-        def _urlfor(*args, **kwargs):
-            import warnings
-            from fresco.core import urlfor
-            warnings.warn(
-                "%r.url() is deprecated. Please use urlfor() instead",
-                DeprecationWarning, stacklevel=2)
-            return urlfor(func, *args, **kwargs)
-        try:
-            func.url = _urlfor
-        except AttributeError:
-            # Cannot set function attributes on bound or unbound methods.
-            # See http://www.python.org/dev/peps/pep-0232/
-            pass
+        Return a copy of the RouteCollection with the given path prepended to
+        all routes.
+        """
+        ob = copy(self)
+        ob.__routes__ = [route.add_prefix(prefix) for route in self]
+        return ob
 
     def pathfor(self, viewspec, *args, **kwargs):
         """\
@@ -895,9 +1011,9 @@ class RouteCollection(object):
             for k in delegated_route.routed_args_default:
                 if k not in factory_kwargs:
                     v = delegated_route.routed_args_default[k]
-                if callable(v):
-                    v = v(request)
-                factory_kwargs[k] = v
+                    if callable(v):
+                        v = v(request)
+                    factory_kwargs[k] = v
             rc = delegated_route.routecollectionfactory(*factory_args,
                                                         **factory_kwargs)
             p2 = rc.pathfor(remainder, request=request, *args, **kwargs)
@@ -988,55 +1104,66 @@ class RouteCollection(object):
             if request and route.predicate and not route.predicate(request):
                 continue
 
-            view_args = route.view_args + result.args
-            view_kwargs = {}
-            if route.view_kwargs:
-                view_kwargs.update(route.view_kwargs)
-            view_kwargs.update(result.kwargs)
+            # View function arguments extracted while traversing the path
+            traversal_args = result.args
+            traversal_kwargs = result.kwargs
 
+            # Process any args/kwargs defined in the Route declaration.
             if request:
-                view_args = tuple(
+                route_args = tuple(
                     (item(request) if isinstance(item, RouteArg) else item)
-                    for item in view_args)
+                    for item in route.view_args)
 
-                for k, v in view_kwargs.items():
-                    if isinstance(v, RouteArg):
-                        view_kwargs[k] = v(request)
+                route_kwargs = dict(
+                    (k, v(request) if isinstance(v, RouteArg) else v)
+                    for k, v in route.view_kwargs.items())
+            else:
+                route_args = route.view_args
+                route_kwargs = route.view_kwargs
+
+            args = route_args + traversal_args
+            kwargs = dict(traversal_kwargs, **route_kwargs)
 
             if isinstance(route, DelegateRoute):
                 try:
-                    sub_routes = route.routecollectionfactory(*view_args,
-                                                              **view_kwargs)
-
+                    sub_routes = route.routecollectionfactory(*args, **kwargs)
                 except ResponseException as e:
                     def raiser(*args, **kwargs):
                         raise e
-                    yield RouteTraversal(self.route_class('/', ALL_METHODS,
-                                                          raiser),
-                                         (), {},
-                                         [(self, '')])
+                    r = self.route_class('/', ALL_METHODS, raiser)
+                    yield RouteTraversal(r, (), {}, [(self, '', r)])
                     continue
-                # Dynamic routes exhaust their arguments when creating the sub
-                # RouteCollection.
-                if route.dynamic:
-                    view_args = ()
-                    view_kwargs = {}
+
                 for sub in sub_routes.get_routes(result.path_remaining,
                                                  method,
                                                  request):
-                    traversed = [(self, '')]
-                    traversed.extend((c, result.path_matched + p)
-                                     for c, p in sub.collections_traversed)
-                    yield RouteTraversal(sub.route,
-                                         view_args + sub.args,
-                                         dict(view_kwargs, **sub.kwargs),
-                                         traversed)
-            else:
-                yield RouteTraversal(route,
-                                     view_args, view_kwargs,
-                                     [(self, '')])
+                    traversed = [TraversedCollection(self, '', route,
+                                                     args,
+                                                     kwargs,
+                                                     traversal_args,
+                                                     traversal_kwargs)]
+                    traversed.extend(
+                        TraversedCollection(c, result.path_matched + p, r,
+                                            a, k, ta, tk)
+                        for c, p, r, a, k, ta, tk in sub.collections_traversed)
 
-    def route(self, pattern, methods, view=None, *args, **kwargs):
+                    # Dynamic routes consume their arguments when creating the
+                    # sub RouteCollection.
+                    if route.dynamic:
+                        yield RouteTraversal(
+                            sub.route, sub.args, sub.kwargs, traversed)
+                    else:
+                        yield RouteTraversal(sub.route,
+                                            args + sub.args,
+                                            dict(kwargs, **sub.kwargs),
+                                            traversed)
+            else:
+                yield RouteTraversal(
+                    route, args, kwargs,
+                    [TraversedCollection(self, '', route, args, kwargs,
+                                         traversal_args, traversal_kwargs)])
+
+    def route(self, pattern, methods=None, view=None, *args, **kwargs):
         """
         Match a URL pattern to a view function. Can be used as a function
         decorator, in which case the ``view`` parameter should not be passed.
@@ -1048,8 +1175,13 @@ class RouteCollection(object):
 
         Other parameters are as for the :class:`Route` constructor.
         """
+        # Catch the common error of not providing a valid method
+        if methods and not isinstance(methods, Iterable):
+            raise TypeError("HTTP methods must be specified as a string "
+                            "or iterable")
+
         # Called as a decorator?
-        if view is None:
+        if methods is not None and view is None:
             def route_decorator(func):
                 self.add_route(self.route_class(pattern, methods, func, *args,
                                                 **kwargs))
@@ -1097,7 +1229,8 @@ class RouteCollection(object):
         """
         routes = list(r.add_prefix(path) for r in views.__routes__)
         for r in routes:
-            r = r.bindto(views)
+            if r.instance is None:
+                r = r.bindto(views)
             self.add_route(r)
 
     def delegate(self, path, app, dynamic=False, *args, **kwargs):
@@ -1169,7 +1302,7 @@ class DelegateRoute(Route):
         Return the RouteCollection responsible for paths under this route
         """
 
-        routes = self.getdecoratedview()(*args, **kwargs)
+        routes = self.getdecoratedview(self.getview(GET))(*args, **kwargs)
 
         if isinstance(routes, RouteCollection):
             return routes
@@ -1177,7 +1310,7 @@ class DelegateRoute(Route):
         return RouteCollection(r.bindto(routes) for r in routes.__routes__)
 
     def _static_routecollectionfactory(self, *args, **kwargs):
-        return self.getview()
+        return self.getview(GET)
 
 
 def register_converter(name, registry=ExtensiblePattern):
