@@ -1,12 +1,28 @@
 # encoding=UTF-8
+# Copyright 2015 Oliver Cope
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
+#
+from __future__ import absolute_import
+
 from fresco import Route, GET, Response
-from fresco.compat import PY3
+from fresco.compat import PY3, ustr
 
 # A unicode path that can't be represented using ASCII encoding
 if PY3:
     unquoted_unicode_path = '/ø'
 else:
-    unquoted_unicode_path = unicode('/ø', 'UTF-8')
+    unquoted_unicode_path = ustr('/ø', 'UTF-8')
 
 # IRI quoted version of the same path. This is the string the server
 # would receive as the HTTP Request-URI
