@@ -1,3 +1,19 @@
+# Copyright 2015 Oliver Cope
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
+#
+from __future__ import absolute_import
+
 from fresco import FrescoApp, GET, Response
 from fresco.decorators import extract_getargs, json_response
 
@@ -17,7 +33,8 @@ class TestExtractGetArgs(object):
             ])
 
         with app.requestcontext('/foo/29'):
-            assert app.view().content == ["Received 'foo':str, 29:int"]
+            assert list(app.view().content_iterator) == \
+                    [b"Received 'foo':str, 29:int"]
 
     def test_query_args(self):
 
@@ -31,7 +48,8 @@ class TestExtractGetArgs(object):
         app = FrescoApp()
         app.route('/', GET, view)
         with app.requestcontext('/?arg1=foo;arg2=29'):
-            assert app.view().content == ["Received 'foo':str, 29:int"]
+            assert list(app.view().content_iterator) == \
+                    [b"Received 'foo':str, 29:int"]
 
     def test_missing_args_with_one_default_argument(self):
 
@@ -60,7 +78,8 @@ class TestExtractGetArgs(object):
         app.route('/', GET, view)
         with app.requestcontext('/?arg1=foo'):
             assert app.view().status == '200 OK'
-            assert app.view().content == ["Received 'foo':str, None:NoneType"]
+            assert list(app.view().content_iterator) == \
+                    [b"Received 'foo':str, None:NoneType"]
 
     def test_conversion_error_without_default(self):
 
@@ -97,23 +116,30 @@ class TestExtractGetArgs(object):
         app = FrescoApp()
         app.route('/', GET, view)
         with app.requestcontext('/?arg1=foo'):
-            assert app.view().content == ['Received None:NoneType']
+            assert list(app.view().content_iterator) == \
+                    [b'Received None:NoneType']
 
 
 class TestJSONResponse(object):
 
-    def test_it_json_encodes(self):
-        r = json_response({'l': [1, 2, 3]})
-        assert r.content == ['{"l":[1,2,3]}']
-        assert r.get_header('content-type') == 'application/json'
-
     def test_it_allows_custom_formatting(self):
         r = json_response({'l': [1, 2, 3]},
                           indent=1, separators=(', ', ': '))
-        assert r.content == ['{\n "l": [\n  1, \n  2, \n  3\n ]\n}']
+        assert r.content_iterator == [b'{\n "l": [\n  1, \n  2, \n  3\n ]\n}']
+
+    def test_it_acts_as_a_filter(self):
+        r = json_response({'l': [1, 2, 3]})
+        assert r.content_iterator == [b'{"l":[1,2,3]}']
+        assert r.get_header('content-type') == 'application/json'
 
     def test_it_acts_as_a_decorator(self):
         @json_response()
         def f():
             return {'l': [1, 2, 3]}
-        assert f().content == ['{"l":[1,2,3]}']
+        assert f().content_iterator == [b'{"l":[1,2,3]}']
+
+    def test_it_acts_as_a_decorator_without_arguments(self):
+        @json_response
+        def f():
+            return {'l': [1]}
+        assert f().content_iterator == [b'{"l":[1]}']
