@@ -1,4 +1,20 @@
-# coding=UTF-8
+# encoding=UTF-8
+# Copyright 2015 Oliver Cope
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
+#
+from __future__ import absolute_import
+
 from io import BytesIO
 
 import pytest
@@ -97,6 +113,12 @@ class TestRequestProperties(object):
                      CONTENT_TYPE='application/broken') as c:
             c.request.get_json(parse_int=lambda s: s + '!') == {'foo': '1!'}
 
+    def test_is_secure_returns_correct_value(self):
+        with context('https://example.org/') as c:
+            assert c.request.is_secure is True
+        with context('http://example.org/') as c:
+            assert c.request.is_secure is False
+
 
 class TestPathEncoding(object):
 
