@@ -1,7 +1,25 @@
-from mock import Mock
+# Copyright 2015 Oliver Cope
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
+#
+from __future__ import absolute_import
 
-from fresco import Route, GET, FrescoApp
+from mock import Mock, call
+import pytest
+
+from fresco import Route, GET, POST, FrescoApp, Response
 from fresco import routeargs
+from fresco.exceptions import BadRequest
 
 
 class TestRouteArg(object):
@@ -15,7 +33,7 @@ class TestRouteArg(object):
 
     def test_routekwarg_value_passed(self):
 
-        view = Mock()
+        view = Mock(return_value=Response())
         routekwarg = Mock(spec=routeargs.RouteArg)
         routekwarg.return_value = 'xyzzy'
 
@@ -23,73 +41,73 @@ class TestRouteArg(object):
         app.route('/', GET, view, x=routekwarg)
         with app.requestcontext('/') as c:
             app.view()
-            routekwarg.assert_called_with(c.request)
-            view.assert_called_with(x='xyzzy')
+            assert routekwarg.call_args_list == [call(c.request)]
+            assert view.call_args_list == [call(x='xyzzy')]
 
     def test_queryarg_value_passed(self):
-        view = Mock()
+        view = Mock(return_value=Response())
         app = FrescoApp()
         app.route('/', GET, view, x=routeargs.QueryArg())
         with app.requestcontext('/?x=foo'):
             app.view()
-            view.assert_called_with(x='foo')
+            assert view.call_args_list == [call(x='foo')]
 
     def test_formarg_value_passed(self):
-        view = Mock()
+        view = Mock(return_value=Response())
         app = FrescoApp()
         app.route('/', GET, view, x=routeargs.FormArg())
         with app.requestcontext('/?x=foo'):
             app.view()
-            view.assert_called_with(x='foo')
+            assert view.call_args_list == [call(x='foo')]
 
     def test_sessionarg_value_passed(self):
-        view = Mock()
+        view = Mock(return_value=Response())
         app = FrescoApp()
         app.route('/', GET, view, x=routeargs.SessionArg())
         with app.requestcontext('/?x=foo') as c:
             c.request.environ[c.request.SESSION_ENV_KEY] = {'x': 'foo'}
             app.view()
-            view.assert_called_with(x='foo')
+            assert view.call_args_list == [call(x='foo')]
 
     def test_cookiearg_value_passed(self):
-        view = Mock()
+        view = Mock(return_value=Response())
         app = FrescoApp()
         app.route('/', GET, view, x=routeargs.CookieArg())
         with app.requestcontext('/', HTTP_COOKIE='x=foo'):
             app.view()
-            view.assert_called_with(x='foo')
+            assert view.call_args_list == [call(x='foo')]
 
     def test_cookiearg_listvalue_passed(self):
-        view = Mock()
+        view = Mock(return_value=Response())
         app = FrescoApp()
         app.route('/', GET, view, x=routeargs.CookieArg([str]))
         with app.requestcontext('/', HTTP_COOKIE='x=foo;x=bar'):
             app.view()
-            view.assert_called_with(x=['foo', 'bar'])
+            assert view.call_args_list == [call(x=['foo', 'bar'])]
 
     def test_requestarg_value_converted(self):
-        view = Mock()
+        view = Mock(return_value=Response())
         app = FrescoApp()
         app.route('/', GET, view, x=routeargs.FormArg(float))
         with app.requestcontext('/?x=0'):
             app.view()
-            view.assert_called_with(x=0.0)
+            assert view.call_args_list == [call(x=0.0)]
 
     def test_requestarg_default_value(self):
-        view = Mock()
+        view = Mock(return_value=Response())
         app = FrescoApp()
         app.route('/', GET, view, x=routeargs.FormArg(default='d'))
         with app.requestcontext('/'):
             app.view()
-            view.assert_called_with(x='d')
+            assert view.call_args_list == [call(x='d')]
 
     def test_it_doesnt_convert_default_values(self):
-        view = Mock()
+        view = Mock(return_value=Response())
         app = FrescoApp()
         app.route('/', GET, view, x=routeargs.FormArg(int, default=None))
         with app.requestcontext('/'):
             app.view()
-            view.assert_called_with(x=None)
+            assert view.call_args_list == [call(x=None)]
 
     def test_access_to_missing_requestarg_returns_badrequest(self):
         def view(x):
@@ -108,26 +126,26 @@ class TestRouteArg(object):
             assert 'Bad Request' in app.view().status
 
     def test_routeargs_work_as_positional_arguments(self):
-        view = Mock()
+        view = Mock(return_value=Response())
         app = FrescoApp()
         app.route('/', GET, view, args=[routeargs.FormArg(key='x')])
         with app.requestcontext('/?x=foo'):
             app.view()
-            view.assert_called_with('foo')
+            assert view.call_args_list == [call('foo')]
 
     def test_routearg_classes_are_auto_instantiated(self):
-        view = Mock()
+        view = Mock(return_value=Response())
         app = FrescoApp()
         app.route('/', GET, view, x=routeargs.FormArg)
         with app.requestcontext('/?x=foo'):
             app.view()
-            view.assert_called_with(x='foo')
+            assert view.call_args_list == [call(x='foo')]
 
 
 class TestRequestObject(object):
 
     def test_it_passes_the_request(self):
-        view = Mock()
+        view = Mock(return_value=Response())
         app = FrescoApp()
         app.route('/', GET, view, x=routeargs.RequestObject())
         with app.requestcontext('/') as c:
@@ -140,7 +158,7 @@ class Test_routearg(object):
 
     def test_it_calls_func(object):
 
-        view = Mock()
+        view = Mock(return_value=Response())
         argfunc = Mock()
         app = FrescoApp()
         app.route('/', GET, view, x=routeargs.routearg(argfunc))
@@ -151,7 +169,7 @@ class Test_routearg(object):
 
     def test_it_passes_additional_args(object):
 
-        view = Mock()
+        view = Mock(return_value=Response())
         argfunc = Mock()
         app = FrescoApp()
         app.route('/', GET, view, x=routeargs.routearg(argfunc, 'x', y=42))
@@ -164,10 +182,43 @@ class Test_routearg(object):
 class TestFormData(object):
 
     def test_it_passes_the_form_dict(self):
-        view = Mock()
+        view = Mock(return_value=Response())
         app = FrescoApp()
         app.route('/', GET, view, x=routeargs.FormData())
         with app.requestcontext('/') as c:
             app.view()
             x = view.call_args[1]['x']
             assert x is c.request.form
+
+
+class TestJSONPayload(object):
+
+    def test_it_passes_json_data(self):
+        view = Mock(return_value=Response())
+        app = FrescoApp()
+        app.route('/', POST, view, x=routeargs.JSONPayload())
+        with app.requestcontext_post('/', data=b'{"foo":"bar"}',
+                                     CONTENT_TYPE='application/json'):
+            app.view()
+            x = view.call_args[1]['x']
+            assert x == {'foo': 'bar'}
+
+    def test_it_raises_bad_request_on_invalid_payload(self):
+
+        view = Mock(return_value=Response())
+        app = FrescoApp()
+        app.route('/', POST, view, x=routeargs.JSONPayload())
+        with app.requestcontext_post('/', data=b'invalid json!',
+                                     CONTENT_TYPE='application/json'):
+            with pytest.raises(BadRequest):
+                app.view()
+
+    def test_it_returns_a_default_value(self):
+        view = Mock(return_value=Response())
+        app = FrescoApp()
+        app.route('/', POST, view,
+                  x=routeargs.JSONPayload(default='default value'))
+        with app.requestcontext_post('/'):
+            app.view()
+        x = view.call_args[1]['x']
+        assert x == 'default value'
