@@ -1,62 +1,144 @@
 # encoding: utf-8
+# Copyright 2015 Oliver Cope
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
+#
+from __future__ import absolute_import
+
 from copy import copy
 from functools import wraps
 
-from mock import Mock
-from flea import Agent
+from mock import Mock, call
 import pytest
 import tms
 
-from fresco import FrescoApp, GET, POST, Response
+from fresco import FrescoApp, ALL_METHODS, GET, POST, Response
 from fresco.exceptions import NotFound
-from fresco.compat import ustring_type
+from fresco.compat import ustr
 from fresco.core import urlfor
 from fresco.routing import (
     Route, DelegateRoute, RouteCollection, routefor, RouteTraversal,
-    RouteNotFound)
+    RouteNotFound, TraversedCollection)
 from . import fixtures
 
 
+def assert_method_bound_to(method, ob):
+    try:
+        assert method.__self__ is ob
+    except AttributeError:
+        assert method.im_self is ob
+
+
 class TestMethodDispatch(object):
 
     def test_route_is_dispatched_to_correct_method(self):
 
-        getview = Mock(return_value=Response(content_type='text/plain'))
-        postview = Mock(return_value=Response(content_type='text/plain'))
+        getview = Mock(return_value=Response())
+        postview = Mock(return_value=Response())
         app = FrescoApp()
         app.route('/', GET, getview)
         app.route('/', POST, postview)
 
-        Agent(app).get('/')
-        assert getview.call_count == 1
-        assert postview.call_count == 0
+        with app.requestcontext('/'):
+            app.view()
+            assert getview.call_count == 1
+            assert postview.call_count == 0
 
-        Agent(app).post('/')
-        assert getview.call_count == 1
-        assert postview.call_count == 1
+        with app.requestcontext_post('/'):
+            app.view()
+            assert getview.call_count == 1
+            assert postview.call_count == 1
+
+
+class TestRouteConstructor(object):
+
+    def test_multiple_views_can_be_associated_with_a_route(self):
+
+        app = FrescoApp()
+        v1 = Mock(return_value=Response())
+        v2 = Mock(return_value=Response())
+        app.route('/', GET=v1, POST=v2)
+
+        with app.requestcontext():
+            app.view()
+            assert v1.call_count == 1
+            assert v2.call_count == 0
+
+        with app.requestcontext_post():
+            app.view()
+            assert v1.call_count == 1
+            assert v2.call_count == 1
+
+    def test_kwargs_take_precedence(self):
+        app = FrescoApp()
+        v1 = Mock(return_value=Response())
+        v2 = Mock(return_value=Response())
+        app.route('/', ALL_METHODS, v1, POST=v2)
+
+        with app.requestcontext():
+            app.view()
+            assert v1.call_count == 1
+            assert v2.call_count == 0
+
+        with app.requestcontext_post():
+            app.view()
+            assert v1.call_count == 1
+            assert v2.call_count == 1
+
+    def test_it_catches_invalid_methods(self):
+
+        # Route must be a string or an iterable
+        with pytest.raises(TypeError):
+            Route('/', object(), lambda: None)
+
+        # Route must be a valid HTTP method
+        with pytest.raises(ValueError):
+            Route('/', 'FOO', lambda: None)
+
+        with pytest.raises(ValueError):
+            Route('/', ['GET', 'FOO'], lambda: None)
 
 
 class TestRouteViewFilters(object):
 
     def exclaim(self, response):
-        return response.replace(content=[''.join(response.content) + '!'])
+        return response.replace(
+            content=[b''.join(response.content_iterator) + b'!'])
 
     def ask(self, response):
-        return response.replace(content=[''.join(response.content) + '?'])
+        return response.replace(
+            content=[b''.join(response.content_iterator) + b'?'])
 
     def test_filter_is_applied(self):
         views = fixtures.CBV('test')
         app = FrescoApp()
         app.route('/', GET, views.index_html).filter(self.ask)
         with app.requestcontext('/'):
-            assert app.view().content == ['test?']
+            assert app.view().content_iterator == [b'test?']
+
+    def test_filter_is_applied_as_route_kwargs(self):
+        views = fixtures.CBV('test')
+        app = FrescoApp()
+        app.route('/', GET, views.index_html, filters=[self.ask])
+        with app.requestcontext('/'):
+            assert app.view().content_iterator == [b'test?']
 
     def test_it_applies_multiple_filters_in_order(self):
         app = FrescoApp()
         views = fixtures.CBV('test')
         app.route('/', GET, views.index_html).filter(self.ask, self.exclaim)
         with app.requestcontext('/'):
-            assert app.view().content == ['test?!']
+            assert app.view().content_iterator == [b'test?!']
 
     def test_it_applies_chained_filter_calls_in_order(self):
         app = FrescoApp()
@@ -64,7 +146,7 @@ class TestRouteViewFilters(object):
         app.route('/', GET, views.index_html)\
                 .filter(self.ask).filter(self.exclaim)
         with app.requestcontext('/'):
-            assert app.view().content == ['test?!']
+            assert app.view().content_iterator == [b'test?!']
 
 
 class TestRouteDecorators(object):
@@ -73,7 +155,8 @@ class TestRouteDecorators(object):
         @wraps(func)
         def exclaim(*args, **kwargs):
             response = func(*args, **kwargs)
-            return response.replace(content=[''.join(response.content) + '!'])
+            return response.replace(
+                content=[b''.join(response.content_iterator) + b'!'])
         return exclaim
 
     def test_decorator_is_applied(self):
@@ -86,10 +169,10 @@ class TestRouteDecorators(object):
         app.route('/plain', GET, views.index_html)
 
         with app.requestcontext('/decorated'):
-            assert app.view().content == ['test!']
+            assert app.view().content_iterator == [b'test!']
 
         with app.requestcontext('/plain'):
-            assert app.view().content == ['test']
+            assert app.view().content_iterator == [b'test']
 
     def test_decorator_is_applied_with_wrap_method(self):
 
@@ -100,10 +183,10 @@ class TestRouteDecorators(object):
         app.route('/plain', GET, views.index_html)
 
         with app.requestcontext('/decorated'):
-            assert app.view().content == ['test!']
+            assert app.view().content_iterator == [b'test!']
 
         with app.requestcontext('/plain'):
-            assert app.view().content == ['test']
+            assert app.view().content_iterator == [b'test']
 
     def test_decorator_works_with_urlfor(self):
 
@@ -139,19 +222,21 @@ class TestPredicates(object):
     def test_predicate_match(self):
 
         def v1():
-            return Response(['x'])
+            return Response([b'x'])
 
         def v2():
-            return Response(['y'])
+            return Response([b'y'])
 
         app = FrescoApp()
         app.route('/', GET, v1, predicate=lambda request: 'x' in request.query)
         app.route('/', GET, v2, predicate=lambda request: 'y' in request.query)
 
-        t = Agent(app)
-        assert t.get('/?x=1', check_status=False).body == 'x'
-        assert t.get('/?y=1', check_status=False).body == 'y'
-        assert t.get('/', check_status=False).response.status_code == 404
+        with app.requestcontext('/?x=1'):
+            assert b''.join(app.view().content) == b'x'
+        with app.requestcontext('/?y=1'):
+            assert b''.join(app.view().content) == b'y'
+        with app.requestcontext('/'):
+            assert app.view().status_code == 404
 
 
 class TestRouteNames(object):
@@ -171,6 +256,30 @@ class TestRouteNames(object):
 
 class TestRouteCollection(object):
 
+    def test_it_adds_routes_from_constructor(self):
+        r1 = Route('/1', GET, None, name='1')
+        r2 = Route('/2', POST, None, name='2')
+        rc = RouteCollection([r1, r2])
+        assert [r.name for r in rc] == ['1', '2']
+
+    def test_it_adds_routecollections_from_constructor(self):
+        r1 = Route('/', GET, None, name='1')
+        r2 = Route('/', POST, None, name='2')
+        r3 = Route('/', POST, None, name='3')
+        rc = RouteCollection([r1, RouteCollection([r2, r3])])
+        assert [r.name for r in rc] == ['1', '2', '3']
+
+    def test_it_adds_dunderroutes_from_constructor(self):
+        r1 = Route('/', GET, None, name='1')
+        r2 = Route('/', POST, None, name='2')
+        r3 = Route('/', POST, None, name='3')
+
+        class A:
+            __routes__ = [r2, r3]
+
+        rc = RouteCollection([r1, A()])
+        assert [r.name for r in rc] == ['1', '2', '3']
+
     def test_get_routes_matches_on_method(self):
 
         r_get = Route('/', GET, None)
@@ -204,26 +313,34 @@ class TestRouteCollection(object):
         a = RouteCollection()
         b = RouteCollection()
 
-        a.route('/harvey', GET, lambda: None)
-        b.route('/harvey', GET, lambda: None)
+        a_route = Route('/harvey', GET, lambda: None)
+        b_route = Route('/harvey', GET, lambda: None)
 
-        a.delegate('/rabbit', b)
-        b.delegate('/hole', a)
+        a.add_route(a_route)
+        b.add_route(b_route)
+
+        a_delegate_route = DelegateRoute('/rabbit', b)
+        b_delegate_route = DelegateRoute('/hole', a)
+
+        a.add_route(a_delegate_route)
+        b.add_route(b_delegate_route)
 
         r = next(a.get_routes('/rabbit/hole/rabbit/harvey', None))
-        assert r.collections_traversed == [(a, ''),
-                                           (b, '/rabbit'),
-                                           (a, '/rabbit/hole'),
-                                           (b, '/rabbit/hole/rabbit')]
+
+        assert r.collections_traversed == [
+            (a, '', a_delegate_route, (), {}, (), {}),
+            (b, '/rabbit', b_delegate_route, (), {}, (), {}),
+            (a, '/rabbit/hole', a_delegate_route, (), {}, (), {}),
+            (b, '/rabbit/hole/rabbit', b_route, (), {}, (), {})]
 
     def test_pathfor_works_with_positional_args(self):
-        view = Mock()
+        view = Mock(return_value=Response())
         rc = RouteCollection([Route('/<:str>', GET, view)])
         assert rc.pathfor(view, 'x') == '/x'
 
     def test_replace_raises_route_not_found(self):
         a = RouteCollection()
-        view = Mock()
+        view = Mock(return_value=Response())
         a.route('/harvey', GET, view, name='harvey')
         with pytest.raises(RouteNotFound):
             a.replace('rabbit', None)
@@ -238,7 +355,7 @@ class TestRouteCollection(object):
 
     def test_replace_selects_routes_by_view(self):
         a = RouteCollection()
-        view = Mock()
+        view = Mock(return_value=Response())
         oldroute = Route('/', GET, view)
         newroute = Route('/', GET, Mock())
         a.add_route(oldroute)
@@ -250,6 +367,16 @@ class TestRouteCollection(object):
         r2 = Route('/', GET, Mock())
         assert (RouteCollection([r1]) + [r2]).__routes__ == [r1, r2]
 
+    def test_can_add_route_to_routecollection(self):
+        r1 = Route('/', GET, Mock())
+        r2 = Route('/', GET, Mock())
+        assert (RouteCollection([r1]) + r2).__routes__ == [r1, r2]
+
+    def test_can_add_routecollection_to_route(self):
+        r1 = Route('/', GET, Mock())
+        r2 = Route('/', GET, Mock())
+        assert (r1 + RouteCollection([r2])).__routes__ == [r1, r2]
+
     def test_can_add_routecollections(self):
         r1 = Route('/', GET, Mock())
         r2 = Route('/', GET, Mock())
@@ -262,12 +389,12 @@ class TestRouteCollection(object):
                 Route('/', GET, 'view')])
 
             def view(self):
-                pass
+                return Response()
 
         v = MyViews()
         app = FrescoApp()
         app.include('/', v)
-        assert [r.route.getview()
+        assert [r.route.getview(GET)
                 for r in app.get_routes('/', GET)] == [v.view]
 
     def test_routecollections_in_classes_can_be_manipulated(self):
@@ -276,21 +403,50 @@ class TestRouteCollection(object):
                 Route('/', GET, 'view')])
 
             def view(self):
-                pass
+                return Response()
 
         class MyOtherViews(MyViews):
             __routes__ = copy(MyViews.__routes__)
             __routes__.replace('view', Route('/', GET, 'another_view'))
 
             def another_view(self):
-                pass
+                return Response()
 
         v = MyOtherViews()
         app = FrescoApp()
         app.include('/', v)
-        assert [r.route.getview()
+        assert [r.route.getview(GET)
                 for r in app.get_routes('/', GET)] == [v.another_view]
 
+    def test_add_prefix_returns_prefixed_collection(self):
+        rc = RouteCollection([Route('/fish', GET, None),
+                              Route('/beans', GET, None)])
+        prefixed = rc.add_prefix('/jelly')
+        assert [str(r.pattern) for r in prefixed] == \
+                ['/jelly/fish', '/jelly/beans']
+
+    def test_it_binds_routes_to_an_instance(self):
+        views = fixtures.CBV('test')
+        rc = RouteCollection([views])
+        view = next(rc.get_routes('/', GET)).route.getview(GET)
+        assert_method_bound_to(view, views)
+
+    def test_it_binds_routes_to_an_instance_via_include(self):
+        views = fixtures.CBV('test')
+        rc = RouteCollection([])
+        rc.include('/', views)
+        view = next(rc.get_routes('/', GET)).route.getview(GET)
+        assert_method_bound_to(view, views)
+
+    def test_including_twice_does_not_rebind_instance(self):
+        views = fixtures.CBV('test')
+        rc = RouteCollection([])
+        rc.include('/', views)
+        rc2 = RouteCollection([])
+        rc2.include('/', rc)
+        view = next(rc2.get_routes('/', GET)).route.getview(GET)
+        assert_method_bound_to(view, views)
+
 
 class TestRoutefor(object):
 
@@ -327,7 +483,7 @@ class TestDelegatedRoutes(object):
     def test_dispatch_to_delegated_route(self):
 
         def hello():
-            return Response(['hello'])
+            return Response([b'hello'])
 
         inner = FrescoApp()
         inner.route('/hello', GET, hello)
@@ -335,12 +491,12 @@ class TestDelegatedRoutes(object):
         outer = FrescoApp()
         outer.delegate('/say', inner)
 
-        r = Agent(outer).get('/say/hello')
-        assert r.body == 'hello'
+        with outer.requestcontext('/say/hello'):
+            assert b''.join(outer.view().content) == b'hello'
 
     def test_url_variables_are_passed(self):
 
-        hello = Mock()
+        hello = Mock(return_value=Response())
 
         inner = FrescoApp()
         inner.route('/<i:str>', GET, hello)
@@ -350,28 +506,29 @@ class TestDelegatedRoutes(object):
 
         with outer.requestcontext('/foo/bar'):
             outer.view()
-            hello.assert_called_with(i='bar', o='foo')
+            assert hello.call_args_list == [call(i='bar', o='foo')]
 
     def test_delegation_to_dynamic_routes(self):
 
         result = []
 
         class MyRoutes(object):
-            __routes__ = [Route('/<inner:int>/view', GET, 'view')]
+            __routes__ = [Route('/<inner:int>', GET, 'view')]
 
             def __init__(self, **kwargs):
                 self.kwargs = kwargs
 
             def view(self, **kwargs):
                 result.append((self, kwargs))
+                return Response()
 
         app = FrescoApp()
         app.delegate('/<outer:str>', MyRoutes, dynamic=True)
-        with app.requestcontext('/two/2/view'):
+        with app.requestcontext('/one/2'):
             app.view()
             instance, inner_kwargs = result[0]
+            assert instance.kwargs == {'outer': 'one'}
             assert inner_kwargs == {'inner': 2}
-            assert instance.kwargs == {'outer': 'two'}
 
     def test_dynamic_routes_are_never_shared(self):
 
@@ -385,6 +542,7 @@ class TestDelegatedRoutes(object):
 
             def view(self):
                 result.append(self.value)
+                return Response()
 
         app = FrescoApp()
         app.delegate('/<value:str>', MyRoutes, dynamic=True)
@@ -410,7 +568,7 @@ class TestDelegatedRoutes(object):
 
     def test_pathfor_with_dynamic_delegated_route(self):
 
-        view = Mock()
+        view = Mock(return_value=Response())
 
         def routecollectionfactory(*args, **kwargs):
             return RouteCollection([Route('/<i:str>', GET, view,
@@ -424,7 +582,7 @@ class TestDelegatedRoutes(object):
 
     def test_pathfor_with_dynamic_delegated_route_uses_default_args(self):
 
-        view = Mock()
+        view = Mock(return_value=Response())
 
         def routecollectionfactory(factoryarg1, factoryarg2):
             return RouteCollection([Route('/<i:str>', GET, view,
@@ -451,6 +609,7 @@ class TestDelegatedRoutes(object):
 
             def view(self, **kwargs):
                 result.append(urlfor(self.view, inner=3))
+                return Response()
 
         app = FrescoApp()
         app.delegate('/<outer:str>', MyRoutes, dynamic=True)
@@ -467,7 +626,7 @@ class TestDelegatedRoutes(object):
                 pass
 
             def view(self, b):
-                pass
+                return Response()
 
         app = FrescoApp()
         app.delegate('/<a:str>', Routable, dynamic=True, name='x')
@@ -476,7 +635,7 @@ class TestDelegatedRoutes(object):
 
     def test_delegated_routes_can_be_included(self):
 
-        view = Mock()
+        view = Mock(return_value=Response())
 
         inner = RouteCollection([Route('/baz', GET, view)])
         middle = RouteCollection([DelegateRoute('/bar', inner)])
@@ -484,7 +643,7 @@ class TestDelegatedRoutes(object):
         outer.include('/foo', middle)
         with outer.requestcontext('/foo/bar/baz'):
             outer.view()
-            view.assert_called()
+            assert view.call_count == 1
 
     def test_not_found_is_returned(self):
 
@@ -501,22 +660,22 @@ class TestDelegatedRoutes(object):
 
         def inner():
             raise NotFound()
-        view = Mock()
+        view = Mock(return_value=Response())
 
         outer = FrescoApp()
         outer.delegate('/foo', inner, dynamic=True)
-        outer.route('/foo', view)
+        outer.route('/foo', GET, view)
         with outer.requestcontext('/foo'):
             outer.view()
-            view.assert_called()
+            assert view.call_count == 1
 
 
 class TestConverters(object):
 
     def test_str_converter_returns_unicode(self):
         from fresco.routing import StrConverter
-        s = ustring_type('abc')
-        assert isinstance(StrConverter().from_string(s), ustring_type)
+        s = ustr('abc')
+        assert isinstance(StrConverter().from_string(s), ustr)
 
     def test_register_coverter_acts_as_decorator(self):
         from fresco.routing import Converter, register_converter
@@ -526,7 +685,7 @@ class TestConverters(object):
             def from_string(self, s):
                 return 'bar'
 
-        view = Mock()
+        view = Mock(return_value=Response())
 
         app = FrescoApp()
         app.route('/<:testconverter>', GET, view)
@@ -553,6 +712,15 @@ class TestViewArgs(object):
         assert list(routes.get_routes('/3', GET)) == \
                 [tms.InstanceOf(RouteTraversal, args=(1, 2, 3))]
 
+    def test_it_keeps_traversal_args_separate(self):
+        routes = RouteCollection([Route('/<:int>', GET, None, view_args=(1,))])
+        assert list(routes.get_routes('/2', GET)) == \
+            [tms.InstanceOf(RouteTraversal, args=(1, 2),
+                            collections_traversed=[
+                                tms.InstanceOf(TraversedCollection,
+                                               args=(1, 2),
+                                               traversal_args=(2,))])]
+
 
 class TestViewKwargs(object):
 
@@ -571,6 +739,17 @@ class TestViewKwargs(object):
         assert list(routes.get_routes('/', GET)) == \
                 [tms.InstanceOf(RouteTraversal, kwargs={'x': 1})]
 
+    def test_it_keeps_traversal_kwargs_separate(self):
+        routes = RouteCollection([
+            Route('/<x:int>', GET, None, view_kwargs={'y': 1})])
+        assert list(routes.get_routes('/2', GET)) == \
+            [tms.InstanceOf(RouteTraversal,
+                            kwargs={'x': 2, 'y': 1},
+                            collections_traversed=[
+                                tms.InstanceOf(TraversedCollection,
+                                               kwargs={'y': 1, 'x': 2},
+                                               traversal_kwargs={'x': 2})])]
+
 
 class TestRouteClassIsPluggable(object):
 
@@ -622,3 +801,59 @@ class TestRouteClassIsPluggable(object):
         assert list(routes2.get_routes('/incl/', GET)) == \
                 [tms.InstanceOf(RouteTraversal,
                                 route=tms.InstanceOf(self.CustomRoute))]
+
+
+class TestRouteTraversal(object):
+
+    routes = RouteCollection()
+    inner = RouteCollection()
+    inner2 = RouteCollection()
+
+    view = object()
+    inner2.route('/<i:int>', GET=view, name='c')
+    inner.delegate('/<i:int>', inner2, name='b')
+    routes.delegate('/<i:int>', lambda i, r=inner: r, dynamic=True, name='a')
+
+    def test_it_reconstructs_the_path(self):
+        traversal = next(self.routes.get_routes('/1/2/3', GET))
+
+        # Do we get the original path back?
+        assert traversal.build_path() == '/1/2/3'
+
+    def test_it_modifies_the_path(self):
+
+        traversal = next(self.routes.get_routes('/1/2/3', GET))
+
+        # Modify the traversal and check we get a modified path back
+        assert traversal.replace('a', {'i': 0}).build_path() == '/0/2/3'
+
+        assert traversal.replace('b', {'i': 0}).build_path() == '/1/0/3'
+        assert traversal.replace('a:b', {'i': 0}).build_path() == '/1/0/3'
+
+        assert traversal.replace('a:b:c', {'i': 0}).build_path() == '/1/2/0'
+        assert traversal.replace('a:c', {'i': 0}).build_path() == '/1/2/0'
+        assert traversal.replace('c', {'i': 0}).build_path() == '/1/2/0'
+        assert traversal.replace(self.view, {'i': 0}).build_path() == '/1/2/0'
+
+        # Passing no arguments should create a new traversal
+        assert traversal.replace('a') is not traversal
+
+        # ...identical to the original
+        assert traversal.replace('a') == traversal
+
+    def test_replace_raises_routenotfound(self):
+        traversal = next(self.routes.get_routes('/1/2/3', GET))
+        with pytest.raises(RouteNotFound):
+            traversal.replace('x')
+
+        with pytest.raises(RouteNotFound):
+            traversal.replace('a:x')
+
+        with pytest.raises(RouteNotFound):
+            traversal.replace('b:a')
+
+        with pytest.raises(RouteNotFound):
+            traversal.replace('a:b:c:d')
+
+        with pytest.raises(RouteNotFound):
+            traversal.replace('')
